/*
 * Copyright (C) 2000 John David Ratliff and Christopher Michael O'Donnell
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

public class BoundedIntegerStack {
	private int maximumStackSize;						// the maximum stack size
	private int[] stackArray;						// the stack itself
	private int stackTop;							// index to the stack top

	/* class constructor - initilize the private member variables */

	public BoundedIntegerStack(int maximumStackSize) {
		this.maximumStackSize = maximumStackSize;			// initilize maximumStackSize
		this.stackArray = new int[maximumStackSize];			// allocate stack array
		this.stackTop = 0;						// initilize index
	}

	/* Push an element onto the stack. If the stack is full, ignore the call; however, since the array size is
		dictated by the size of the String, the stack will never actually be full.
	*/

	public void push(int element) {
		if (this.isFull() == false) {					// is the stack full?
			this.stackArray[this.stackTop] = element;		// if no, 
			this.stackTop++;
			this.stackArray[this.stackTop] = Integer.MIN_VALUE;
		}
	}

	/* Pop the top element off the stack. If the stack is empty, return the value Integer.MIN_VALUE (- 2^32 / 2)
		Integer.MIN_VALUE is assumed to be a 'null' value on the stack.
	*/

	public int pop() {
		int element;							// element to pop

		if (this.isEmpty() == true) {					// is the stack empty?
			return Integer.MIN_VALUE;				// yes: return Integer.MIN_VALUE (- 2^32 / 2)
		} else {							//
			this.stackTop--;					// no: decrement the stack pointer
			element = this.stackArray[this.stackTop];		// pop the element off the stack
			this.stackArray[this.stackTop] = Integer.MIN_VALUE;	// delete the element from the stack
		}

		return element;							// return the element
	}

	/* Return the size of the stack -- since the stack is a linear array, the size is simply the index pointer */

	public int size() {
		return this.stackTop;
	}

	/* Check to see if the stack is full by seeing if the size of the array equals the maximumStackSize */

	public boolean isFull() {
		int size = this.size();						// assign stack size to size variable

		if (size == this.maximumStackSize) {				// is stack full?
			return true;						// return true
		} else {							// else
			return false;						// return false
		}
	}

	/* Check to see if the stack is empty. The stack is considered to be empty if the size of the array is 0 */

	public boolean isEmpty() {
		int size = this.size();						// assign stack size to size variable

		if (size == 0) {						// is stack empty?
			return true;						// return true
		} else {							// else
			return false;						// return false
		}
	}
}

// End of BoundedIntegerStack class
