/*
 * Copyright (C) 2000 John David Ratliff and Christopher Michael O'Donnell
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package tools;

import datastructures.BoundedIntegerStack;
import java.util.StringTokenizer;

public class PostfixIntegerExpressionParser {
	private BoundedIntegerStack stack;				// the Integer stack
	private String string;						// the string to parse
	private int intermediateResult;					// the intermediate result of computations

	/* the class constructor: initilize the private member variables (all except the stack which is initilized in
		this.parse()
	*/

	public PostfixIntegerExpressionParser(String string) {
		this.string = string;					// initilize string
		this.intermediateResult = 0;				// initilize the intermediate result to 0
	}

	/* Check to see if the character passed is an operator (+ - * /) */

	private boolean isOperator(char character) {
		boolean operator;					// operator boolean (is it or isn't it?)

		switch (character) {					// switch statement for operator tests
			case '+':					// did we get a +, -, *, or / passed?
			case '-':
			case '*':
			case '/':
				operator = true;			// if yes, then we got an operator
				break;					// end case statement

			default:					// for anything other than +, -, *, and /
				operator = false;			// we did NOT get an operator
				break;					// end the case statement
		}							// end of switch

		return operator;					// return the operator
	}

	/* Takes two operands and the operation and computes the intermediate result */

	private void compute(int operand1, int operand2, char operator) {
		switch (operator) {					// store result of operation in intermediateResult
			case '+':
				this.intermediateResult = operand1 + operand2;
				break;

			case '-':
				this.intermediateResult = operand1 - operand2;
				break;

			case '*':
				this.intermediateResult = operand1 * operand2;
				break;

			case '/':
				this.intermediateResult = operand1 / operand2;
				break;

			default:					// shouldn't ever get this, but it's nice to be clean
				break;
		}							// end switch statement
	}

	/* Parse the string. If we find numbers, push them onto the stack. If we find operators, pop the top 2 stack items
		and call the compute method. Push the intermediateResult back onto the stack as the top item.
	*/

	public void parse() {
		StringTokenizer tokens;					// The tokens of the string
		String token;						// String variable for the individual token
		Integer intWrapper;					// integer wrapper to get a primitive from
		char operator;						// the operator, if we get one
		int stackSize;						// size of the stack
		int op1, op2;						// the two operands to compute
		int element;						// the element to be pushed on the stack

		tokens = new StringTokenizer(this.string);		// Tokenize the string
		stackSize = tokens.countTokens();			// The stackSize will be the number of tokens
		this.stack = new BoundedIntegerStack(stackSize);	// create stack of size stackSize

		while (tokens.hasMoreTokens()) {			// while we have more tokens
			token = tokens.nextToken();			// Put the new token into the token String
			operator = token.charAt(0);			// Grab the first character to test for operators

			if (this.isOperator(operator)) {		// Did we get an operator?
				op2 = stack.pop();			// If so, pop the last 2 elements from the stack
				op1 = stack.pop();
				this.compute(op1,op2,operator);		// and compute them
				stack.push(this.intermediateResult);	// now stick the intermediateResult on the stack
			} else {					// not an operator?
				intWrapper = new Integer(token);	// Create an Integer instance from the String token
				element = intWrapper.intValue();	// Now get a primitive from the intWrapper
				stack.push(element);			// Push that element back onto the stack
			}
		}
	}

	/* Return the last intermediateResult from parsing the string as the result of the Calculator */

	public int evaluate() {
		this.parse();						// parse the string

		return this.intermediateResult;				// return the intermediateResult
	}
}
